@@ -238,8 +238,7 @@ int librados::RadosClient::connect()
     return -EISCONN;
   state = CONNECTING;
 
-  if (cct->_conf->log_early &&
-      !cct->_log->is_started()) {
+  if (!cct->_log->is_started()) {
     cct->_log->start();
   }
 
