@@ -156,9 +156,8 @@ class MDCache {
 
   Filer filer;
 
-  bool exceeded_size_limit = false;
-
 private:
+  void upkeep_main(void);
   uint64_t cache_inode_limit;
   uint64_t cache_memory_limit;
   double cache_reservation;
