@@ -34,9 +34,9 @@ void bluestore_bdev_label_t::encode(bufferlist& bl) const
   ENCODE_FINISH(bl);
 }
 
-void bluestore_bdev_label_t::decode(bufferlist::iterator& p)
+void bluestore_bdev_label_t::decode(bufferlist::const_iterator& p)
 {
-  p.advance(60); // see above
+  p.advance(60u); // see above
   DECODE_START(2, p);
   decode(osd_uuid, p);
   decode(size, p);
@@ -107,9 +107,9 @@ void bluestore_extent_ref_map_t::_check() const
   unsigned refs = 0;
   for (const auto &p : ref_map) {
     if (p.first < pos)
-      ceph_assert(0 == "overlap");
+      ceph_abort_msg("overlap");
     if (p.first == pos && p.second.refs == refs)
-      ceph_assert(0 == "unmerged");
+      ceph_abort_msg("unmerged");
     pos = p.first + p.second.length;
     refs = p.second.refs;
   }
@@ -197,11 +197,11 @@ void bluestore_extent_ref_map_t::put(
   auto p = ref_map.lower_bound(offset);
   if (p == ref_map.end() || p->first > offset) {
     if (p == ref_map.begin()) {
-      ceph_assert(0 == "put on missing extent (nothing before)");
+      ceph_abort_msg("put on missing extent (nothing before)");
     }
     --p;
     if (p->first + p->second.length <= offset) {
-      ceph_assert(0 == "put on missing extent (gap)");
+      ceph_abort_msg("put on missing extent (gap)");
     }
   }
   if (p->first < offset) {
@@ -358,6 +358,10 @@ void bluestore_blob_use_tracker_t::allocate()
 {
   ceph_assert(num_au != 0);
   bytes_per_au = new uint32_t[num_au];
+  mempool::get_pool(
+    mempool::pool_index_t(mempool::mempool_bluestore_cache_other)).
+      adjust_count(1, sizeof(uint32_t) * num_au);
+
   for (uint32_t i = 0; i < num_au; ++i) {
     bytes_per_au[i] = 0;
   }
@@ -1000,6 +1004,8 @@ void bluestore_blob_t::split(uint32_t blob_offset, bluestore_blob_t& rb)
 }
 
 // bluestore_shared_blob_t
+MEMPOOL_DEFINE_OBJECT_FACTORY(bluestore_shared_blob_t, bluestore_shared_blob_t,
+	          bluestore_cache_other);
 
 void bluestore_shared_blob_t::dump(Formatter *f) const
 {
