@@ -9,7 +9,7 @@ import { ConfirmationModalComponent } from '../../../shared/components/confirmat
 import { CriticalConfirmationModalComponent } from '../../../shared/components/critical-confirmation-modal/critical-confirmation-modal.component';
 import { ActionLabelsI18n } from '../../../shared/constants/app.constants';
 import { TableComponent } from '../../../shared/datatable/table/table.component';
-import { CellTemplate } from '../../../shared/enum/cell-template.enum';
+import { Icons } from '../../../shared/enum/icons.enum';
 import { ViewCacheStatus } from '../../../shared/enum/view-cache-status.enum';
 import { CdTableAction } from '../../../shared/models/cd-table-action';
 import { CdTableColumn } from '../../../shared/models/cd-table-column';
@@ -48,6 +48,8 @@ export class RbdListComponent implements OnInit {
   nameTpl: TemplateRef<any>;
   @ViewChild('flattenTpl')
   flattenTpl: TemplateRef<any>;
+  @ViewChild('removingStatTpl')
+  removingStatTpl: TemplateRef<any>;
 
   permission: Permission;
   tableActions: CdTableAction[];
@@ -56,6 +58,7 @@ export class RbdListComponent implements OnInit {
   retries: number;
   viewCacheStatusList: any[];
   selection = new CdTableSelection();
+  icons = Icons;
 
   modalRef: BsModalRef;
 
@@ -107,7 +110,9 @@ export class RbdListComponent implements OnInit {
       permission: 'update',
       icon: 'fa-pencil',
       routerLink: () => this.urlBuilder.getEdit(getImageUri()),
-      name: this.actionLabels.EDIT
+      name: this.actionLabels.EDIT,
+      disable: () => !this.selection.first() || !_.isUndefined(this.getRemovingStatusDesc()),
+      disableDesc: () => this.getRemovingStatusDesc()
     };
     const deleteAction: CdTableAction = {
       permission: 'delete',
@@ -119,18 +124,25 @@ export class RbdListComponent implements OnInit {
       permission: 'create',
       canBePrimary: (selection: CdTableSelection) => selection.hasSingleSelection,
       disable: (selection: CdTableSelection) =>
-        !selection.hasSingleSelection || selection.first().cdExecuting,
+        !selection.hasSingleSelection ||
+        selection.first().cdExecuting ||
+        !_.isUndefined(this.getRemovingStatusDesc()),
       icon: 'fa-copy',
       routerLink: () => `/block/rbd/copy/${getImageUri()}`,
-      name: this.actionLabels.COPY
+      name: this.actionLabels.COPY,
+      disableDesc: () => this.getRemovingStatusDesc()
     };
     const flattenAction: CdTableAction = {
       permission: 'update',
       disable: (selection: CdTableSelection) =>
-        !selection.hasSingleSelection || selection.first().cdExecuting || !selection.first().parent,
+        !selection.hasSingleSelection ||
+        selection.first().cdExecuting ||
+        !selection.first().parent ||
+        !_.isUndefined(this.getRemovingStatusDesc()),
       icon: 'fa-chain-broken',
       click: () => this.flattenRbdModal(),
-      name: this.actionLabels.FLATTEN
+      name: this.actionLabels.FLATTEN,
+      disableDesc: () => this.getRemovingStatusDesc()
     };
     const moveAction: CdTableAction = {
       permission: 'delete',
@@ -140,7 +152,9 @@ export class RbdListComponent implements OnInit {
       disable: (selection: CdTableSelection) =>
         !selection.first() ||
         !selection.hasSingleSelection ||
-        selection.first().image_format === RBDImageFormat.V1
+        selection.first().image_format === RBDImageFormat.V1 ||
+        !_.isUndefined(this.getRemovingStatusDesc()),
+      disableDesc: () => this.getRemovingStatusDesc()
     };
     this.tableActions = [
       addAction,
@@ -158,7 +172,7 @@ export class RbdListComponent implements OnInit {
         name: this.i18n('Name'),
         prop: 'name',
         flexGrow: 2,
-        cellTransformation: CellTemplate.executing
+        cellTemplate: this.removingStatTpl
       },
       {
         name: this.i18n('Pool'),
@@ -350,4 +364,11 @@ export class RbdListComponent implements OnInit {
 
     this.modalRef = this.modalService.show(ConfirmationModalComponent, { initialState });
   }
+
+  getRemovingStatusDesc(): string | undefined {
+    const first = this.selection.first();
+    if (first && first.source && first.source === 'REMOVING') {
+      return this.i18n(`Action not possible for an RBD in status 'Removing'`);
+    }
+  }
 }
