@@ -1,4 +1,4 @@
-import { Component, OnInit, ViewChild } from '@angular/core';
+import { Component, OnInit } from '@angular/core';
 
 import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
 
@@ -12,8 +12,6 @@ import { AboutComponent } from '../about/about.component';
   styleUrls: ['./dashboard-help.component.scss']
 })
 export class DashboardHelpComponent implements OnInit {
-  @ViewChild('docsForm')
-  docsFormElement;
   docsUrl: string;
   modalRef: BsModalRef;
 
@@ -41,8 +39,4 @@ export class DashboardHelpComponent implements OnInit {
   openAboutModal() {
     this.modalRef = this.modalService.show(AboutComponent);
   }
-
-  goToApiDocs() {
-    this.docsFormElement.nativeElement.submit();
-  }
 }
