@@ -187,4 +187,26 @@ describe('PrometheusAlertService', () => {
       expect(notificationService.show).toHaveBeenCalledTimes(2);
     });
   });
+
+  describe('alert badge', () => {
+    beforeEach(() => {
+      service = TestBed.get(PrometheusAlertService);
+
+      prometheusService = TestBed.get(PrometheusService);
+      spyOn(prometheusService, 'ifAlertmanagerConfigured').and.callFake((fn) => fn());
+      spyOn(prometheusService, 'getAlerts').and.callFake(() => of(alerts));
+
+      alerts = [
+        prometheus.createAlert('alert0', 'active'),
+        prometheus.createAlert('alert1', 'suppressed'),
+        prometheus.createAlert('alert2', 'suppressed')
+      ];
+      service.refresh();
+    });
+
+    it('should count active alerts', () => {
+      service.refresh();
+      expect(service.activeAlerts).toBe(1);
+    });
+  });
 });
