@@ -21,7 +21,7 @@
 #include "rgw_auth_s3.h"
 
 #include "common/ceph_crypto_cms.h"
-#include "common/armor.h"
+#include "common/ceph_crypto.h"
 #include "common/Cond.h"
 
 #define dout_subsys ceph_subsys_rgw
@@ -380,6 +380,141 @@ EC2Engine::get_from_keystone(const DoutPrefixProvider* dpp, const boost::string_
   return std::make_pair(std::move(token_envelope), 0);
 }
 
+std::pair<boost::optional<std::string>, int> EC2Engine::get_secret_from_keystone(const DoutPrefixProvider* dpp,
+                                                                                 const std::string& user_id,
+                                                                                 const boost::string_view& access_key_id) const
+{
+  /*  Fetch from /users/{USER_ID}/credentials/OS-EC2/{ACCESS_KEY_ID} */
+  /* Should return json with response key "credential" which contains entry "secret"*/
+
+  /* prepare keystone url */
+  std::string keystone_url = config.get_endpoint_url();
+  if (keystone_url.empty()) {
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  const auto api_version = config.get_api_version();
+  if (config.get_api_version() == rgw::keystone::ApiVersion::VER_3) {
+    keystone_url.append("v3/");
+  } else {
+    keystone_url.append("v2.0/");
+  }
+  keystone_url.append("users/");
+  keystone_url.append(user_id);
+  keystone_url.append("/credentials/OS-EC2/");
+  keystone_url.append(access_key_id.to_string());
+
+  /* get authentication token for Keystone. */
+  std::string admin_token;
+  int ret = rgw::keystone::Service::get_admin_token(cct, token_cache, config,
+                                                    admin_token);
+  if (ret < 0) {
+    ldpp_dout(dpp, 2) << "s3 keystone: cannot get token for keystone access"
+                  << dendl;
+    return make_pair(boost::none, ret);
+  }
+
+  using RGWGetAccessSecret
+    = rgw::keystone::Service::RGWKeystoneHTTPTransceiver;
+
+  /* The container for plain response obtained from Keystone.*/
+  ceph::bufferlist token_body_bl;
+  RGWGetAccessSecret secret(cct, "GET", keystone_url, &token_body_bl);
+
+  /* set required headers for keystone request */
+  secret.append_header("X-Auth-Token", admin_token);
+
+  /* check if we want to verify keystone's ssl certs */
+  secret.set_verify_ssl(cct->_conf->rgw_keystone_verify_ssl);
+
+  /* send request */
+  ret = secret.process();
+  if (ret < 0) {
+    ldpp_dout(dpp, 2) << "s3 keystone: secret fetching error: "
+                  << token_body_bl.c_str() << dendl;
+    return make_pair(boost::none, ret);
+  }
+
+  /* if the supplied signature is wrong, we will get 401 from Keystone */
+  if (secret.get_http_status() ==
+          decltype(secret)::HTTP_STATUS_NOTFOUND) {
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  /* now parse response */
+
+  JSONParser parser;
+  if (! parser.parse(token_body_bl.c_str(), token_body_bl.length())) {
+    ldpp_dout(dpp, 0) << "Keystone credential parse error: malformed json" << dendl;
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  JSONObjIter credential_iter = parser.find_first("credential");
+  std::string secret_string;
+
+  try {
+    if (!credential_iter.end()) {
+      JSONDecoder::decode_json("secret", secret_string, *credential_iter, true);
+    } else {
+      ldpp_dout(dpp, 0) << "Keystone credential not present in return from server" << dendl;
+      return make_pair(boost::none, -EINVAL);
+    }
+  } catch (JSONDecoder::err& err) {
+    ldpp_dout(dpp, 0) << "Keystone credential parse error: " << err.message << dendl;
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  return make_pair(secret_string, 0);
+}
+
+/*
+ * Try to get a token for S3 authentication, using a secret cache if available
+ */
+std::pair<boost::optional<rgw::keystone::TokenEnvelope>, int>
+EC2Engine::get_access_token(const DoutPrefixProvider* dpp,
+			    const boost::string_view& access_key_id,
+                            const std::string& string_to_sign,
+                            const boost::string_view& signature,
+			    const signature_factory_t& signature_factory) const
+{
+  using server_signature_t = VersionAbstractor::server_signature_t;
+  boost::optional<rgw::keystone::TokenEnvelope> token;
+  int failure_reason;
+
+  /* Get a token from the cache if one has already been stored */
+  boost::optional<boost::tuple<rgw::keystone::TokenEnvelope, std::string>>
+    t = secret_cache.find(access_key_id.to_string());
+
+  /* Check that credentials can correctly be used to sign data */
+  if (t) {
+    std::string sig(signature);
+    server_signature_t server_signature = signature_factory(cct, t->get<1>(), string_to_sign);
+    if (sig.compare(server_signature) == 0) {
+      return std::make_pair(t->get<0>(), 0);
+    } else {
+      ldpp_dout(dpp, 0) << "Secret string does not correctly sign payload, cache miss" << dendl;
+    }
+  } else {
+    ldpp_dout(dpp, 0) << "No stored secret string, cache miss" << dendl;
+  }
+
+  /* No cached token, token expired, or secret invalid: fall back to keystone */
+  std::tie(token, failure_reason) = get_from_keystone(dpp, access_key_id, string_to_sign, signature);
+
+  if (token) {
+    /* Fetch secret from keystone for the access_key_id */
+    boost::optional<std::string> secret;
+    std::tie(secret, failure_reason) = get_secret_from_keystone(dpp, token->get_user_id(), access_key_id);
+
+    if (secret) {
+      /* Add token, secret pair to cache, and set timeout */
+      secret_cache.add(access_key_id.to_string(), *token, *secret);
+    }
+  }
+
+  return std::make_pair(token, failure_reason);
+}
+
 EC2Engine::acl_strategy_t
 EC2Engine::get_acl_strategy(const EC2Engine::token_envelope_t&) const
 {
@@ -424,7 +559,7 @@ rgw::auth::Engine::result_t EC2Engine::authenticate(
   const boost::string_view& signature,
   const boost::string_view& session_token,
   const string_to_sign_t& string_to_sign,
-  const signature_factory_t&,
+  const signature_factory_t& signature_factory,
   const completer_factory_t& completer_factory,
   /* Passthorugh only! */
   const req_state* s) const
@@ -447,7 +582,7 @@ rgw::auth::Engine::result_t EC2Engine::authenticate(
   boost::optional<token_envelope_t> t;
   int failure_reason;
   std::tie(t, failure_reason) = \
-    get_from_keystone(dpp, access_key_id, string_to_sign, signature);
+    get_access_token(dpp, access_key_id, string_to_sign, signature, signature_factory);
   if (! t) {
     return result_t::deny(failure_reason);
   }
@@ -486,6 +621,63 @@ rgw::auth::Engine::result_t EC2Engine::authenticate(
   }
 }
 
+bool SecretCache::find(const std::string& token_id,
+                       SecretCache::token_envelope_t& token,
+		       std::string &secret)
+{
+  std::lock_guard<std::mutex> l(lock);
+
+  map<std::string, secret_entry>::iterator iter = secrets.find(token_id);
+  if (iter == secrets.end()) {
+    return false;
+  }
+
+  secret_entry& entry = iter->second;
+  secrets_lru.erase(entry.lru_iter);
+
+  const utime_t now = ceph_clock_now();
+  if (entry.token.expired() || now > entry.expires) {
+    secrets.erase(iter);
+    return false;
+  }
+  token = entry.token;
+  secret = entry.secret;
+
+  secrets_lru.push_front(token_id);
+  entry.lru_iter = secrets_lru.begin();
+
+  return true;
+}
+
+void SecretCache::add(const std::string& token_id,
+                      const SecretCache::token_envelope_t& token,
+		      const std::string& secret)
+{
+  std::lock_guard<std::mutex> l(lock);
+
+  map<string, secret_entry>::iterator iter = secrets.find(token_id);
+  if (iter != secrets.end()) {
+    secret_entry& e = iter->second;
+    secrets_lru.erase(e.lru_iter);
+  }
+
+  const utime_t now = ceph_clock_now();
+  secrets_lru.push_front(token_id);
+  secret_entry& entry = secrets[token_id];
+  entry.token = token;
+  entry.secret = secret;
+  entry.expires = now + s3_token_expiry_length;
+  entry.lru_iter = secrets_lru.begin();
+
+  while (secrets_lru.size() > max) {
+    list<string>::reverse_iterator riter = secrets_lru.rbegin();
+    iter = secrets.find(*riter);
+    assert(iter != secrets.end());
+    secrets.erase(iter);
+    secrets_lru.pop_back();
+  }
+}
+
 }; /* namespace keystone */
 }; /* namespace auth */
 }; /* namespace rgw */
