@@ -3,12 +3,9 @@
 
 #include "rgw_rest.h"
 
-class RGWRESTMgr_PubSub_S3 : public RGWRESTMgr {
-  RGWRESTMgr *next;
+class RGWRESTMgr_PubSub : public RGWRESTMgr {
 public:
-  explicit RGWRESTMgr_PubSub_S3(RGWRESTMgr *_next) : next(_next) {}
-
-  RGWHandler_REST *get_handler(struct req_state* s,
+  virtual RGWHandler_REST* get_handler(struct req_state* s,
                                const rgw::auth::StrategyRegistry& auth_registry,
                                const std::string& frontend_prefix) override;
 };
